import dash
from dash.exceptions import PreventUpdate
from dash.dependencies import Input, Output, State
import json

from dash_app.app import app
from dash_app.graph import Graph
from dash_app.words import AssociatedWords


word2vec_model = AssociatedWords()


@app.callback(
    Output(component_id='msx-graph-div', component_property='children'),
    Input(component_id='submit-word-button', component_property='n_clicks'),
)
def update_graph_div(submit_word_button):
    graph = Graph()
    graph.set_nodes_and_edges({'nodes': [], 'edges': []})
    return graph.get_cytoscape_graph('msx-graph')


@app.callback(
    Output(component_id='base-word-div', component_property='children'),
    Input(component_id='submit-word-button', component_property='n_clicks'),
    State(component_id='base-word-input', component_property='value'),
)
def update_base_word(submit_word_button, base_word_input):
    return base_word_input


@app.callback(
    Output(component_id='graph-elements-div', component_property='children'),
    Input(component_id='submit-word-button', component_property='n_clicks'),
    Input(component_id='add-word-button', component_property='n_clicks'),
    Input(component_id='extend-graph-button', component_property='n_clicks'),
    Input(component_id='remove-word-button', component_property='n_clicks'),
    State(component_id='base-word-input', component_property='value'),
    State(component_id='add-word-input', component_property='value'),
    State(component_id='graph-elements-div', component_property='children'),
    State(component_id='msx-graph', component_property='tapNodeData'),
    State(component_id='base-word-div', component_property='children'),
    prevent_initial_call=True
)
def update_graph_elements(submit_word_button, add_word_button, extend_graph_button, remove_word_button, base_word_input,
                          add_word_input, nodes_and_edges, selected_nodes, base_word_state):

    callback_context = dash.callback_context
    button_id = callback_context.triggered[0]['prop_id'].split('.')[0]

    graph = Graph()

    if button_id == 'submit-word-button':
        graph.fill_with_associations(word2vec_model, base_word_input)
        new_nodes_and_edges = graph.get_nodes_and_edges()
        return json.dumps(new_nodes_and_edges)

    if button_id == 'add-word-button':
        graph.set_nodes_and_edges(json.loads(nodes_and_edges))
        if add_word_input is not None and add_word_input != '' and add_word_input not in graph.get_all_words():
            graph.add_node(add_word_input)
            graph.add_edge(base_word_state, add_word_input)
            new_nodes_and_edges = json.dumps(graph.get_nodes_and_edges())
            return new_nodes_and_edges
        else:
            raise PreventUpdate

    if button_id == 'extend-graph-button':
        graph.set_nodes_and_edges(json.loads(nodes_and_edges))
        graph.extend_graph(word2vec_model, base_word_state)
        new_nodes_and_edges = graph.get_nodes_and_edges()
        return json.dumps(new_nodes_and_edges)

    if button_id == 'remove-word-button':
        graph.set_nodes_and_edges(json.loads(nodes_and_edges))
        if selected_nodes is not None:
            selected_word = selected_nodes['label']
            if selected_word in graph.get_all_words() and selected_word != base_word_state:
                graph.remove_node(selected_nodes['label'])
                new_nodes_and_edges = json.dumps(graph.get_nodes_and_edges())
                return new_nodes_and_edges
            else:
                raise PreventUpdate
        else:
            raise PreventUpdate


@app.callback(
    Output(component_id='msx-graph', component_property='elements'),
    Input(component_id='graph-elements-div', component_property='children'),
    prevent_initial_call=True
)
def update_graph(nodes_and_edges):
    graph = Graph()
    graph.set_nodes_and_edges(json.loads(nodes_and_edges))
    return graph.get_elements()


@app.callback(
    Output(component_id='msx-graph', component_property='autoRefreshLayout'),
    Input(component_id='add-word-button', component_property='n_clicks'),
    Input(component_id='extend-graph-button', component_property='n_clicks'),
    Input(component_id='remove-word-button', component_property='n_clicks'),
    prevent_initial_call=True
)
def set_auto_refresh_layout(add_word_button, extend_graph_button, remove_word_button):
    callback_context = dash.callback_context
    button_id = callback_context.triggered[0]['prop_id'].split('.')[0]
    if button_id == 'add-word-button' or button_id == 'remove-word-button':
        return False
    if button_id == 'extend-graph-button':
        return True
